#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#define NR_ARGUMENTS_TESTBATCH         ((int) 6)


static const char *pcUsage = 
  "%s usage:\n  %s <input-filename> <output-filename> ./tse <io-filename> <test-batch>\n";
static pid_t startScriptProcess     (   char *pcTseFilename,
                                        char *arg1,
                                        char *arg2);
static pid_t startFileCompareProcess(   char *pcInputFilname, 
                                        char *pcOutputFilename);

int main( int argc, char* argv[])
{
  int    iResult;
  int    iStatus;
  int    iError;
  pid_t  tPidScript;
  pid_t  tPidCompare;
  pid_t  tPidWait;

  iResult = 0;
  if (argc != NR_ARGUMENTS_TESTBATCH)
  {
    printf(pcUsage,argv[0],argv[0]);
    iResult = -1;
  }
  else
  {
    tPidScript = startScriptProcess(argv[3], argv[4], argv[5]);
    if (-1 != tPidScript)
    {
      tPidCompare = startFileCompareProcess(argv[1], argv[2]);
      if (-1 == tPidCompare)
      {
        kill(tPidScript, SIGINT);
        iResult = -1;
      }
      else
      {
        printf("tPidScript = %d, tPidCompare = %d\n", tPidScript,tPidCompare);
      }
    }
    else
    {
      iResult = -1;
    }
    if (0 == iResult)
    {
      sleep(1);
      iError = 0;
      do
      {
        tPidWait = wait(&iStatus);
        if (tPidWait != -1)
        {
          if (tPidWait == tPidScript)
          {
            printf("Script ended\n");
            kill(tPidCompare, SIGKILL);
            iError = -1;
          }
          else if (tPidWait == tPidCompare)
          {
            printf("Difference script ended\n");
            kill(tPidScript, SIGINT);
            iError = -1;
          }
          else
          {
            printf("Waiting for childs but unknown child ended quiting\n");
            kill(tPidScript, SIGINT);
            kill(tPidCompare, SIGKILL);
            iError = -1;
            iResult = -1;
          }
        }
        else
        {
          if (errno != EINTR)
          {
            printf("Script ended\n");
            kill(tPidScript, SIGINT);
            kill(tPidCompare, SIGINT);
            iError = -1;
            iResult = -1;
          } 
        }
      }
      while ( (tPidWait == -1) && (iError == 0));
    }
  }
  return(iResult);
}

static pid_t startScriptProcess(char *pcTseFilename, char *pcArg1, char *pcArg2)
{
  int iResult;
  pid_t tPid;
  char  *ppcBatchArguments[4];

  tPid = fork();
  if (tPid == -1)
  {
    iResult = -1;
  }
  else if (tPid == 0)
  {
    ppcBatchArguments[0] = pcTseFilename;
    ppcBatchArguments[1] = pcArg1;
    ppcBatchArguments[2] = pcArg2;
    ppcBatchArguments[3] = NULL;
    
    iResult = execvp(pcTseFilename,ppcBatchArguments);
    if (iResult == -1)
    {
      printf("Failed to start tse %s\n", strerror(errno)); 
    }
    exit(iResult);
/*    iResult = startChildScriptProcess(pcScriptFilename); */
  }
  else
  {
    iResult = tPid;
  }
  return (tPid);
}

static pid_t startFileCompareProcess(char *pcInputFilname, char *pcOutputFilename)
{
  int    iResult;
  pid_t  tPid;
  char  *ppcFilenames[4];

  tPid = fork();
  if (tPid == -1)
  {
    iResult = -1;
  }
  else if (tPid == 0)
  {
    printf("Starting file diff\n");
    iResult = 0;
    ppcFilenames[0] = "./filediff.sh";
    ppcFilenames[1] = pcInputFilname;
    ppcFilenames[2] = pcOutputFilename;
    ppcFilenames[3] = NULL;
    
    iResult = execvp("./filediff.sh", ppcFilenames);
    if (iResult == -1)
    {
      printf("Failed to start Diff %s\n", strerror(errno)); 
    }
/*    iResult = startChildFileCompareProcess(pcInputFilname, pcOutputFilename); */
    exit(iResult);
  }
  else
  {
    iResult = tPid;
  }
  return (tPid);
  
}

