//#  DH_MSMake.h: DataHolder for the MS info
//#
//#  Copyright (C) 2005
//#  ASTRON (Netherlands Foundation for Research in Astronomy)
//#  P.O.Box 2, 7990 AA Dwingeloo, The Netherlands, seg@astron.nl
//#
//#  This program is free software; you can redistribute it and/or modify
//#  it under the terms of the GNU General Public License as published by
//#  the Free Software Foundation; either version 2 of the License, or
//#  (at your option) any later version.
//#
//#  This program is distributed in the hope that it will be useful,
//#  but WITHOUT ANY WARRANTY; without even the implied warranty of
//#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//#  GNU General Public License for more details.
//#
//#  You should have received a copy of the GNU General Public License
//#  along with this program; if not, write to the Free Software
//#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//#
//#  $Id$

#ifndef BB_MS_DH_MSINFO_H
#define BB_MS_DH_MSINFO_H

#include <Transport/DataHolder.h>
#include <Transport/TH_MPI.h>
#include <Transport/Connection.h>
#include <vector>

//# Forward Declarations.
namespace casa {
  template<class T> class Array;
}


namespace LOFAR {

  class DH_MSMake: public DataHolder
  {
  public:
    DH_MSMake (const string& dhName);
    DH_MSMake (const DH_MSMake&);
    virtual ~DH_MSMake();
    virtual DH_MSMake* clone() const;
    virtual void init();
    virtual void fillDataPointers();

    void setFreq (double start, double end, int n)
      { itsFreqs[0]=start; itsFreqs[1]=end; *itsNFreq=n; }
    void setTime (double start, double end, int n)
      { itsTimes[0]=start; itsTimes[1]=end; *itsNTime=n; }
    void setPos (double ra, double dec)
      { itsPos[0]=ra; itsPos[1]=dec; }
    void fillExtra (const string& msName, const casa::Array<double>& antPos);

    void getFreq (double& start, double& end, int& n) const
      { start=itsFreqs[0]; end=itsFreqs[1]; n=*itsNFreq; }
    void getTime (double& start, double& end, int& n) const
      { start=itsTimes[0]; end=itsTimes[1]; n=*itsNTime; }
    void getPos (double& ra, double& dec) const
      { ra=itsPos[0]; dec=itsPos[1]; }
    void getExtra (string& msName, casa::Array<double>& antPos);

  private:
    // Forbid assignment.
    DH_MSMake& operator= (const DH_MSMake&);

    int*    itsNFreq;
    int*    itsNTime;
    double* itsFreqs;
    double* itsTimes;
    double* itsPos;
  };


  struct MSMakeConn
  {
    MSMakeConn (int nslave);
    ~MSMakeConn();
    DH_MSMake sender;
    DH_MSMake receiver;
#ifdef HAVE_MPI
    std::vector<TH_MPI*> ths;
#endif
    std::vector<Connection*> conns;
  };
}

#endif
