//  Block.cc:
//
//  Copyright (C) 2000, 2001
//  ASTRON (Netherlands Foundation for Research in Astronomy)
//  P.O.Box 2, 7990 AA Dwingeloo, The Netherlands, seg@astron.nl
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  $Id$
//
//
//////////////////////////////////////////////////////////////////////

#include <lofar_config.h>

#include <CEPFrame/Block.h>
#include <CEPFrame/Composite.h>

namespace LOFAR
{

Block::Block (const Block& that)
{
  itsRep = that.itsRep;
  if (itsRep) {
    itsRep->incrRefCount();
  }
}

Block::Block (BlockRep* rep)
{
  itsRep = rep;
  if (itsRep) {
    itsRep->incrRefCount();
  }
}

Block& Block::operator= (const Block& that)
{
  if (this != &that) {
    if (itsRep  &&  itsRep->decrRefCount() <= 0) {
      delete itsRep;
    }
    itsRep = that.itsRep;
    if (itsRep) {
      itsRep->incrRefCount();
    }
  }
  return *this;
}

Block::~Block() 
{
  if (itsRep  &&  itsRep->decrRefCount() <= 0) {
    delete itsRep;
  }
}

Composite Block::getParent() const
{
  return Composite(itsRep->getParent());
}

}
